#include <JUI/Widgets/ScrollingRect.hpp>


using namespace JUI;

void ScrollingRect::RecomputeRenderTarget() {
    InnerDraw();
}

void ScrollingRect::Draw() {

    // TODO: Only recompute when something has changed.
    // Q: How do we know when something has changed?
    RecomputeRenderTarget();

    J2D::DrawPartialRenderTarget(canvas, GetAbsolutePosition(), {0, 0}, Vector2(GetAbsoluteSize().x, GetAbsoluteSize().y));

    bool canvas_larger_than_widget = scroll_size > GetAbsoluteSize().y;
    float ratio = scroll_size / GetAbsoluteSize().y;

    if (vertical_scrollbar_enabled && canvas_larger_than_widget) {
        Vector2 vscroll_pos = GetAbsolutePosition() + Vector2(GetAbsoluteSize().x-scrollbar_width, 0);
        Vector2 vscroll_size = {scrollbar_width, GetAbsoluteSize().y-scrollbar_width};
        J2D::FillRect(Colors::LightGray, vscroll_pos, vscroll_size);
        J2D::FillRoundedRect(scrollbar_color, {vscroll_pos.x, vscroll_pos.y + scroll}, {vscroll_size.x, (vscroll_size.y / ratio)}, 4);

        // Draw little square box at the point where the vertical and horizontal scrollbars would meet.
        Vector2 lil_box_size = {scrollbar_width, scrollbar_width};
        Vector2 lil_box_pos = GetAbsolutePosition() + GetAbsoluteSize() - lil_box_size;
        J2D::FillRect(Colors::DarkGray, lil_box_pos, lil_box_size);
    }
}

Vector2 ScrollingRect::CanvasPosition() const {
    return {GetAbsolutePosition().x, GetAbsolutePosition().y - scroll};
}

// Wouldn't inner draw actually render everything onto the RenderTarget?
void ScrollingRect::InnerDraw() {

    J2D::Begin(canvas, true);
        auto saved_pos = position;
        auto saved_size = size;

        // Kind of a hack to get the child widgets onto the rendertarget at the correct position.
        // We basically set the position for just long enough to lie to the child widgets.
        // The amount we offset by is the absolute position.
        // If you work the formulas out, the child widgets will believe their parent's AbsolutePosition is {0,0}.

        Position(UDim2::FromPixels(-GetAbsolutePosition().x, -GetAbsolutePosition().y + scroll));
        Size({200_percent, 500_percent});

        Rect::InnerDraw();
        DrawChildWidgets();

        J2D::DrawPoint(Colors::Blue, {1,1}, 2);
        J2D::DrawPoint(Colors::Blue, Vector2(GetAbsoluteSize()), 2);

        // Set our position back once we're done.
        Size(saved_size);
        Position(saved_pos);
    J2D::End();

}

const Vector2i default_initialize_canvas_size {1024, 4096};

ScrollingRect::~ScrollingRect() { delete canvas; }

ScrollingRect::ScrollingRect() : canvas(new RenderTarget(default_initialize_canvas_size)) {
    Name("ScrollingRect");
    //bool success = canvas->SetMSAAEnabled(JGL::SampleRate::X8);
}

ScrollingRect::ScrollingRect(Widget *parent) : Rect(parent), canvas(new RenderTarget(default_initialize_canvas_size)) {
    //bool success = canvas->SetMSAAEnabled(JGL::MSAA_SAMPLE_RATE::MSAA_8X);
}

JGL::RenderTarget *ScrollingRect::GetCanvas() { return canvas; }

Vector2i ScrollingRect::CanvasSize() const { return canvas->GetDimensions(); }
